/* This file was generated by syncqt. */
#ifndef QT_QTDOCGALLERY_VERSION_H
#define QT_QTDOCGALLERY_VERSION_H

#define QTDOCGALLERY_VERSION_STR "5.0.0"

#define QTDOCGALLERY_VERSION 0x050000

#endif // QT_QTDOCGALLERY_VERSION_H
